"use strict";
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.$ = void 0;
var react_1 = require("react");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var shared_1 = require("./shared");
// TODO: add better TS support
/**
 * <$> fragment will subscribe to it's Observable children and display
 * it's emissions along with regular children
 *
 * e.g.
 *
 * ```jsx
 * function App(){
 *   return <$>{ timer(0, 1000) }</$>    // 0, 1, 2, 3, ...
 * }
 * ```
 */
function $(props) {
    var children = props === null || props === void 0 ? void 0 : props.children;
    // CHORTCUT:
    // if fragment has many children -- we render it with a <> that has
    // <$> children in place of Observables
    if (Array.isArray(children)) {
        return react_1.createElement.apply(void 0, __spreadArrays([react_1.Fragment, null], children.map(function (c) { return rxjs_1.isObservable(c) ? react_1.createElement($, null, c) : c; })));
    }
    // Single child:
    // state for Observable children
    var _a = react_1.useState(null), streamChild = _a[0], setStreamChild = _a[1];
    // store error indicator
    var _b = react_1.useState(null), error = _b[0], setError = _b[1];
    var destroy$ = shared_1.useDestroyObservable();
    // react to child updates
    react_1.useEffect(function () {
        if (!rxjs_1.isObservable(children)) {
            return;
        }
        // child is a single observable
        // if the stream emits async - synchronously reset child to null
        // else - use value from the stream to update the child
        var syncChildValue = null;
        var isSync = true;
        var sub = children.pipe(operators_1.distinctUntilChanged(), operators_1.takeUntil(destroy$)).subscribe({
            next: function (value) {
                // synchronous values would be set in one run
                if (isSync) {
                    syncChildValue = value;
                }
                else {
                    setStreamChild(value);
                }
            },
            error: function (error) {
                // wrap error in an object to be safe in case the error is nullish
                setError({ error: error });
            },
        });
        isSync = false;
        // make the sync update
        setStreamChild(syncChildValue);
        // clear subscription if Observable child changes
        return function () { return sub.unsubscribe(); };
    }, [children]);
    // raise an error if Observable failed
    if (error) {
        throw error.error;
    }
    return rxjs_1.isObservable(children)
        ? streamChild // read child updates from state
        : children; // child is a regular child, like you and me
}
exports.$ = $;
